@@ -1,7 +1,7 @@
 /**
  * Draw background from tiles
 
- * Copyright (C) 2007  Sylvain Beucler
+ * Copyright (C) 2007, 2008  Sylvain Beucler
 
  * This file is part of GNU FreeDink
 
@@ -26,11 +26,23 @@
 /* #include <ddraw.h> */
 #include "SDL.h"
 
-#define NB_TILE_SCREENS 41
+#define GFX_TILES_NB_SETS 41
 
-/* extern LPDIRECTDRAWSURFACE tiles[]; */
-/* extern RECT tilerect[]; */
-extern SDL_Surface *GFX_tiles[];
+/* 96 = 12 * 8 tile squares; 1 tile square = 50x50 pixels */
+#define GFX_TILES_SCREEN_W 12
+#define GFX_TILES_SCREEN_H 8
+#define GFX_TILES_PER_SCREEN (GFX_TILES_SCREEN_W * GFX_TILES_SCREEN_H)
+#define GFX_TILES_SQUARE_SIZE 50
+#define GFX_TILES_NB_SQUARES (128*(GFX_TILES_NB_SETS-1)+GFX_TILES_SCREEN_W*GFX_TILES_SCREEN_H)
+
+extern SDL_Surface *gfx_tiles[GFX_TILES_NB_SETS + 1];
+
+/* Background square in a screen */
+struct tile
+{
+  short square_full_idx0; /* tile index */
+  short althard; /* alternate hardness index, 0 = default tile hardness */
+};
 
 extern void tiles_load_default(void);
 extern void tiles_load_slot(char* relpath, int slot);
@@ -38,5 +50,7 @@ extern void tiles_unload_all(void);
 extern void draw_map_game(void);
 extern void draw_map_game_background(void);
 extern void process_animated_tiles(void);
+extern void gfx_tiles_draw_screen();
+extern void gfx_tiles_draw(int srctileset_idx0, int srctile_square_idx0, int dsttile_square_idx0);
 
 #endif
